/*
IBM Confidential

OCO Source Materials

5630-A36

(c) Copyright IBM Corp. 1992, 2002

The source code for this program is not published or other-
wise divested of its trade secrets, irrespective of what has
been deposited with the U.S. Copyright Office.
*/

/*                     Labeler Exit                    Labeler.c
 *
 * This module provides a simple ICAPI routine which shows how to supply
 * labels to ICS. It looks at the name of the service requested, and
 * can return "labels" for the RSACi system and for the Coolness system.
 * I put "labels" in quotes, 'cuz we aren't actually generating realy labels;
 * they're syntactically valid labels, but we assign the same rating to
 * each and every page in the universe (which is semantic hogwash).
 *
 * Written 4/25/97 by Martin Presler-Marshall
 *
 * To build on OS/2: (using VisualAge C++)
 *     icc /Ge- /Gm+ /c labeler.c
 *     ILINK /NOFREE Labeler.obj, Labeler.dll, Labeler.map, httpdapi.lib, Labeler.def;
 * [httpdapi.lib is supplied with ICS]
 *
 * To build on Windows NT, compile Labeler.c, link with HTTPDAPI.lib, and use
 * module-definition file LabelerW.def.
 *
 * To build on AIX:
 *       cc_r -c Labeler.c
 *       cc_r -bM:SRE -bnoentry -o libLabeler.o Labeler.o -bI:libhttpdapi.exp -bE:Labeler.exp
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "HTAPI.h" /* Included with ICS. */

/* Rating service names and templates for labels.
 * These will be used to assemble outgoing labels.
 *
 * Note: none of these strings should ever be translated.
 */

    /* RSACi */
char *RSAC_Label_partA =
    "(PICS-1.1 \"http://www.rsac.org/ratingsv01.html\" l for \"";
char *RSAC_Label_partB =
    "\" r (n 0 s 0 v 0 l 0))";
char *RSAC_Name =
    "http://www.rsac.org/ratingsv01.html";

    /* Coolness */
char *Cool_Label =
    "(PICS-1.1 \"http://coolness.raleigh.ibm.com/ratings/V1.html\" l r (Coolness 1 Age-range 1 Graphics 1))";
char *Cool_Name =
    "http://coolness.raleigh.ibm.com/ratings/V1.html";

/* Names of variables we'll extract from ICS using ICAPI.
 * PICS_SERVICENAME = the name of the rating service that a label is
 *     wanted from.
 * PICS_SITENAME    = the protocol & hostname part of URL that a label is
 *     needed for.
 * PICS_PATHNAME    = the path/filename part of the URL that a label is
 *     needed for.
 */
char *SVCNAME  = "PICS_SERVICENAME";
char *SITENAME = "PICS_SITENAME";
char *PATHNAME = "PICS_PATHNAME";

/* Helper functions to manage strings in the heap. We use a brain-dead method
 * of dynamically building labels; I do not suggest this memory management
 * scheme unless pain is something you enjoy.
 */
static char *mystrdup(char *s) /* Make a copy of a string, in the heap. */
{
    char *ret = malloc(strlen(s) + 1);
    if (ret) {
        strcpy(ret, s);
    }
    return(ret);
}

static char *mystrcat(char *a, char *b) /* Glue together two strings; free the first. */
{
    char * ret = malloc(strlen(a) + strlen(b) + 1);
    if (ret) {
        strcpy(ret, a);
        strcat(ret, b);
    }
    free(a);
    return(ret);
}

/* Aha, finally.
 * This is the label-supplier exit. ICS will call this function when the
 * proxy server needs a label to make a filtering decision. If you compile
 * this module to "Labeler.dll", then you'd put the following into the ICS
 * config file (httpd.cnf/httpd.conf):
 *
 * OS/2, NT:
 *    PicsDBLookup                     Labeler.dll:BogusLabelSupplier
 * AIX:
 *    PicsDBLookup                     libLabeler.o:BogusLabelSupplier
 *
 * On OS/2 and NT, if Labeler.dll isn't in the directory you start ICS from,
 * then put a full pathname on it. On AIX, put libLabeler.o in /usr/lib, or
 * give a full path to the file.
 */
void HTTPD_LINKAGE BogusLabelSupplier(unsigned char *handle, long *return_code)
{
    char ReturnBuffer[1024];/* A string buffer to hold results from ICAPI functions. */
    char *URL = NULL;       /* The URL ICS is examining. */
    char *Service = NULL;   /* The name (URL) of the rating service ICS is considering. */
    unsigned long l1, l2;   /* Lengths of various things. */
    long retc;              /* Stores return code from ICAPI functions. */

    /* Determine the URL in question. This is in two steps: the protocol/host
     * part, and the path/file part.
     */
    l1 = strlen(SITENAME);
    l2 = 1023;
    HTTPD_extract(handle,
        SITENAME, /* Fetch protocol/site part of the URL. */
        &l1,
        ReturnBuffer,
        &l2,
        &retc);
    if (retc != HTTPD_SUCCESS) {
        *return_code = 500;
        goto DropOut;
    }
    ReturnBuffer[l2] = '\0';
    URL = mystrdup(ReturnBuffer);
    if (!URL) {
        *return_code = 500;
        goto DropOut;
    }

    l1 = strlen(PATHNAME);
    l2 = 1023;
    HTTPD_extract(handle,
        PATHNAME, /* Fetch path/file part of the URL. */
        &l1,
        ReturnBuffer,
        &l2,
        &retc);
    if (retc != HTTPD_SUCCESS) {
        *return_code = HTTP_NOACTION; /* Let others (if any) try to satisfy this request. */
        goto DropOut;
    }
    ReturnBuffer[l2] = '\0';
    /* And now assemble the full URL. */
    URL = mystrcat(URL, ReturnBuffer);
    if (!URL) {
        *return_code = HTTP_NOACTION; /* Let others (if any) have a shot. */
        goto DropOut;
    }

    /* Determine what service this label is for. */
    l1 = strlen(SVCNAME);
    l2 = 1023;
    HTTPD_extract(handle,
        SVCNAME,
        &l1,
        ReturnBuffer,
        &l2,
        &retc);
    if (retc != HTTPD_SUCCESS) {
        *return_code = HTTP_NOACTION;
        goto DropOut;
    }
    /* NUL-terminate the string... */
    ReturnBuffer[l2] = '\0';
    /* ...and make a copy of it. */
    Service = mystrdup(ReturnBuffer);
    if (!Service) {
        *return_code = HTTP_NOACTION;
        goto DropOut;
    }

    /* At this point, we should actually generate a real label. Since this
     * is an example program, we just return a hard-coded (fake) label
     * above. But we'll at least insert the URL into the label before we
     * send it...if it's an RSACi label.
     */

    if (!strcmp(ReturnBuffer, RSAC_Name)) {
        char *RSAC_Label = mystrdup(RSAC_Label_partA);
        if (RSAC_Label) RSAC_Label = mystrcat(RSAC_Label, URL);
        if (RSAC_Label) RSAC_Label = mystrcat(RSAC_Label, RSAC_Label_partB);
        if (!RSAC_Label) {
            *return_code = HTTP_SERVER_ERROR; /* At this point, we know we should've handled the request - but we couldn't. */
            goto DropOut;
        }
        l1 = strlen(RSAC_Label);
        HTTPD_supply_label(handle,
            RSAC_Label,
            &l1,
            &retc);
        if (retc != HTTPD_SUCCESS) {
            *return_code = HTTP_SERVER_ERROR;
            return;
        }
        *return_code = HTTP_OK;
    } else if (!strcmp(ReturnBuffer, Cool_Name)) {
        /* The Cool_Label doesn't need to be formatted as it doesn't
         * contain the URL...so just transmit it as-is.
         */
        l1 = strlen(Cool_Label);
        HTTPD_supply_label(handle,
            Cool_Label,
            &l1,
            &retc);
        if (retc != HTTPD_SUCCESS) {
            *return_code = HTTP_SERVER_ERROR;
            goto DropOut;
        }
        *return_code = HTTP_OK;

    } else {
        /* Label requested from an unknown service...we'll just indicate
         * that we have no knowledge.
         */
        *return_code = HTTP_NOACTION;
    }

DropOut:
    /* Free anything that we've got left lying around on the heap. */
    if (URL) free(URL);
    if (Service) free(Service);

    /* And return. ICS will examine the value of "return_code" to tell
     * if we were able to provide a label.
     */
    return;
}
